function compile_model(libpbpPath)

  mexName = 'denoiseBP';
  headerFileName = 'DenoisePotential.hpp';
  className = 'DenoisePotential';

  addpath(fullfile(libpbpPath, 'matlab'));

  %%% define symbolic unary and binary potentials %%%%%%%%%%%%%%%%%%%%%%%

  % clear all symbolic variable properties from previous run:
  reset(symengine); % reset symbolic solver engine

  x = sym('x', [1 1]);
  y = sym('y', [1 1]);
  d = sym('img[n+1]');
  assumeAlso(d, 'real');
  alpha = sym('alpha', [3, 1]);
  assume(alpha > 0);
  symUnaryPot = alpha(1) * (x-d)*(x-d);
  symBinaryPot = alpha(2) * symmin((x-y)*(x-y), alpha(3));

  parameters = {...
    'img', 'alpha'};

  %%% generate c++ header files %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  generateGMPotentialHeader(...
    headerFileName, ...
    className, ...
    symUnaryPot, ...
    symBinaryPot, ...
    'parameters', parameters);

   %%% generate mex file %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   generateGMmex(...
     mexName, ...
     headerFileName, ...
     className, ...
     parameters);

  %%% compile mex file %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  compileGMmex(mexName, libpbpPath);

end

function s = symmin(a, b)

  b1 = feval(symengine, '_exprseq', a > b, b);
  b2 = feval(symengine, '_exprseq', a <= b, a);
  s = feval(symengine, 'piecewise', b1, b2);

end
