function denoiseMRF(libpbpPath, InputImgFile, OutputImgFile, nIterBP, nIterMCMC, ...
  nParticles, T_0, T_N, alpha)

  close all;

  addpath(fullfile(libpbpPath, 'matlab'));


  % Load image:
  img = double(imread(InputImgFile))/255;
  img(:) = max(0, min(1, img(:)));
  img = squeeze(mean(img, 3));
  [H, W] = size(img);



  nNodes = W*H;
  % Make edgeStruct
  horEdges = zeros(2, (W-1)*H);
  for x=1:W-1
    for y=1:H
      k = y + (x-1) * H;
      horEdges(1, k) = y + (x-1) * H;
      horEdges(2, k) = y + x * H;
    end
  end
  vertEdges = zeros(2, W*(H-1));
  for x=1:W
    for y=1:H-1
      k = y + (x-1) * (H-1);
      vertEdges(1, k) = y + (x-1) * H;
      vertEdges(2, k) = y+1 + (x-1) * H;
    end
  end
  edgeEnds = uint32([horEdges, vertEdges]);

  % Make 'checkerboard' node schedule:
  pattern = checkerboard(1, H/2+1, W/2+1);
  pattern = pattern(1:H, 1:W);
  nodes1 = uint32(find(pattern == 0)-1);
  nodes2 = uint32(find(pattern == 1)-1);
  nodeSchedule = {nodes1, nodes2};

  % Make initial particles:
  x = zeros(1, nParticles, nNodes);
  for p = 1:nParticles
    x(1, p, :) = img(:);
  end

  % Call MEX function created by compile_model.m
  [x, nlBel, ~] = denoiseBP(...
    edgeEnds, ...
    nNodes, ...
    nodeSchedule, ...
    x, [], [], ...
    nIterMCMC, ...
    temperatureSchedule(T_0, T_N, nIterBP), ...
    img, alpha);

  % Compute nodeLabels
  [~, nodeIDs] = min(nlBel,[], 1);
  nodeLabels = x(:, nodeIDs(1:nNodes) + nParticles * double(0:nNodes-1));

  % Display
  result = reshape(nodeLabels, H, W);
  imwrite(result, OutputImgFile);
  imshow(result);

end

function schedule = temperatureSchedule(T_0, T_N, nIter)
  schedule = T_0 * (T_N / T_0) .^ min(1, double(0:(nIter-1))/nIter);
end

