/* BeliefPropagation.hpp
 * 
 * Copyright: Oliver Müller (TNT, Leibniz Universität Hannover), 2013
 * E-Mail: omueller@tnt.uni-hannover.de
 */
#ifndef PBP_BELIEF_PROPAGATION_HPP
#define PBP_BELIEF_PROPAGATION_HPP

#include <VisitingSchedule.hpp>
#include <MemoryBuffer.hpp>
#include <GraphicalModel.hpp>
#include <StateSpace.hpp>
#include <GMPotential.hpp>

namespace pbp {

  typedef Array<Array<Energy> > MessageBuffer;
  class Messages : public MemoryBuffer<MessageBuffer> {
  public:
    Messages(StateSpace& stateSpace);
    Messages(StateSpace& stateSpace, MessageBuffer oldBufVec, MessageBuffer currentBufVec);
  private:
    MessageBuffer createBuf(StateSpace& stateSpace);
    MessageBuffer validateBuf(StateSpace& stateSpace, MessageBuffer& buf);
  };

  typedef Array<Array<Energy> > BeliefBuffer;
  class Beliefs : public MemoryBuffer<BeliefBuffer> {
  public:
    Beliefs(StateSpace& stateSpace);
    Beliefs(StateSpace& stateSpace, BeliefBuffer oldBufVec, BeliefBuffer currentBufVec);
  private:
    BeliefBuffer createBuf(StateSpace& stateSpace);
    BeliefBuffer validateBuf(StateSpace& stateSpace, BeliefBuffer& buf);
  };

  class BeliefPropagation {
  public:

    /** constructor. */
    BeliefPropagation(VisitingSchedule& visitingSchedule,
              Messages& nlMsg,
              Beliefs& nlBel,
              GraphicalModel& graphicalModel,
              StateSpace& stateSpace,
              GMPotential& gmPotential
    );

    /** run single Belief Propagation iteration */
    virtual void iterate();

    /** Access methods: */
    const Messages& getNlMsg() const { return nlMsg; };
    const Beliefs& getNlBel() const { return nlBel; };
  private:
    BeliefPropagation();
  protected:
    VisitingSchedule &visitingSchedule;
    Messages &nlMsg;
    Beliefs &nlBel;
    GraphicalModel &graphicalModel;
    StateSpace &stateSpace;
    GMPotential &gmPotential;

    std::vector<Energy> collectPrevMessages(Index node_t, Index edge_from_s_to_t);
    Energy calcNewMessage(const Label stateX, const std::vector<Energy> &Mall_ts, const Index t, const Index edge_from_s_to_t, const Index nEdges);
    void normalize(Index node);

  };

}
#endif
