/* GMPotential.hpp
 * 
 * Copyright: Oliver Müller (TNT, Leibniz Universität Hannover), 2013
 * E-Mail: omueller@tnt.uni-hannover.de
 */
#ifndef PBP_GM_POTENTIAL_HPP
#define PBP_GM_POTENTIAL_HPP

#include <pbpTypes.hpp>
#include <Interval.hpp>

namespace pbp {

  class GMPotential {
    public:
      virtual Energy nodePotential(Label x, Index n) const = 0;
      virtual Interval nodeInterval(Label x, Index n, Index dimension, Energy U) const = 0;
      virtual Energy edgePotential(Label x, Label y, Index e) const = 0;
      virtual Interval edgeXInterval(Label x, Label y, Index e, Index dimension, Energy U) const = 0;
      virtual Interval edgeYInterval(Label x, Label y, Index e, Index dimension, Energy U) const = 0;
      virtual ~GMPotential() {};
  };

  class DebugPotentialWrapper : public GMPotential {
  public:
    DebugPotentialWrapper(std::ostream &os, GMPotential &target) : os(os), target(target) {};
    virtual Energy nodePotential(Label x, Index n) const {
      #pragma omp critical (Array)
        os << "nodePotential(x=" << x << ", n=" << n << ")";
      Energy result = target.nodePotential(x, n);
      #pragma omp critical (Array)
        os << " = " << result << std::endl;
      return result;
    };
    virtual Interval nodeInterval(Label x, Index n, Index dimension, Energy U) const {
      #pragma omp critical (Array)
        os << "nodeInterval(x=" << x << ", n=" << n << ", dim=" << dimension << ", U=" << U << ")";
      Interval result = target.nodeInterval(x, n, dimension, U);
      #pragma omp critical (Array)
        os << " = " << result << std::endl;
      return result;
    };
    virtual Energy edgePotential(Label x, Label y, Index e) const {
      #pragma omp critical (Array)
        os << "edgePotential(x=" << x << ", y=" << y << ", e=" << e << ")";
      Energy result = target.edgePotential(x, y, e);
      #pragma omp critical (Array)
        os << " = " << result << std::endl;
      return result;
    }
    virtual Interval edgeXInterval(Label x, Label y, Index e, Index dimension, Energy U) const {
      #pragma omp critical (Array)
        os << "edgeXInterval(x=" << x << ", y=" << y << ", e=" << e << ", dim=" << dimension << ", U=" << U << ")";
      Interval result = target.edgeXInterval(x, y, e, dimension, U);
      #pragma omp critical (Array)
        os << " = " << result << std::endl;
      return result;
    };
    virtual Interval edgeYInterval(Label x, Label y, Index e, Index dimension, Energy U) const {
      #pragma omp critical (Array)
        os << "edgeYInterval(x=" << x << ", y=" << y << ", e=" << e << ", dim=" << dimension << ", U=" << U << ")";
      Interval result = target.edgeYInterval(x, y, e, dimension, U);
      #pragma omp critical (Array)
        os << " = " << result << std::endl;
      return result;
    };
  private:
    std::ostream &os;
    GMPotential &target;
  };

}
#endif
