/* GraphicalModel.hpp
 * 
 * Copyright: Oliver Müller (TNT, Leibniz Universität Hannover), 2013
 * E-Mail: omueller@tnt.uni-hannover.de
 */
#ifndef PBP_GRAPHICAL_MODEL_HPP
#define PBP_GRAPHICAL_MODEL_HPP

#include <pbpTypes.hpp>
#include <vector>

namespace pbp {

  class GraphicalModel {
  public:
    struct Edge {
      Index start;
      Index end;
      Edge(Index start, Index end) : start(start), end(end) {};
      friend std::ostream& operator<<(std::ostream& os, const GraphicalModel::Edge& edge) {
        os << edge.start << "--" << edge.end;
        return os;
      };
    };

    GraphicalModel(Array<Edge> edges, Index nNodes);

    inline Index getNumberOfNodes() const { return nNodes; };
    inline Index getNumberOfEdges() const { return E1.size() / 2; };
    inline const std::vector<Index>& getV() const { return V; };
    inline const std::vector<Index>& getE1() const { return E1; };
    inline const std::vector<Index>& getE2() const { return E2; };
    inline const std::vector<Index>& getN2() const { return N2; };

  private:
    std::vector<Index> E1;
    std::vector<Index> E2;
    std::vector<Index> N2;
    std::vector<Index> V;
    Index nNodes;
  };
  
}
#endif
