/* Interval.hpp
 * 
 * Copyright: Oliver Müller (TNT, Leibniz Universität Hannover), 2013
 * E-Mail: omueller@tnt.uni-hannover.de
 */
#ifndef PBP_INTERVAL_HPP
#define PBP_INTERVAL_HPP

#define MAX_RANGE 30

#include <iostream>

namespace pbp {

  class Interval {
    public:
      /** default constructor: Create empty interval. */
      Interval();
      Interval(double left, double right);

      void clear();
      Interval& set_union(double left, double right);
      Interval& set_union(const Interval &intvl);
      Interval& intersect(double left, double right);
      Interval& intersect(const Interval &intvl);

      double sampleUniform();
      bool isEmpty();
      bool isInInterval(double val);
      std::ostream& write(std::ostream& os) const;
    private:
      int nRange;
      double Start[MAX_RANGE];
      double End[MAX_RANGE];
      void delete_entry(int index);
      void insert_entry(int index, double left, double right);
      void error(const std::string msg);
  };

  std::ostream& operator<<(std::ostream& os, const Interval& intvl);

}
#endif
