/* MemoryBuffer.hpp
 * 
 * Copyright: Oliver Müller (TNT, Leibniz Universität Hannover), 2013
 * E-Mail: omueller@tnt.uni-hannover.de
 */
#ifndef PBP_MEMORY_BUFFER_HPP
#define PBP_MEMORY_BUFFER_HPP

namespace pbp {
  
  template<class T>
  class MemoryBuffer {
  public:
    /* Default constructor: */
    MemoryBuffer() {};
    MemoryBuffer(T &&oldBuf, T &&currentBuf) : oldBufPriv(oldBuf), currentBufPriv(currentBuf) {};

    /* Access methods: */
    T& currentBuf() { return currentBufPriv; };
    const T& currentBuf() const { return currentBufPriv; };
    T& oldBuf() { return oldBufPriv; };
    const T& oldBuf() const { return oldBufPriv; };

    /* Swap current & old buffers: */
    void swap() {
      using std::swap;
      swap(oldBufPriv, currentBufPriv);
      currentBufPriv = oldBufPriv;
    };
  private:
    MemoryBuffer(const MemoryBuffer &memoryBuffer) {};

    T oldBufPriv;
    T currentBufPriv;
  };

}
#endif
