/* SlicePBP.hpp
 * 
 * Copyright: Oliver Müller (TNT, Leibniz Universität Hannover), 2013
 * E-Mail: omueller@tnt.uni-hannover.de
 */
#ifndef PBP_SLICE_PBP_HPP
#define PBP_SLICE_PBP_HPP

#include <VisitingSchedule.hpp>
#include <MemoryBuffer.hpp>
#include <GraphicalModel.hpp>
#include <StateSpace.hpp>
#include <GMPotential.hpp>
#include <BeliefPropagation.hpp>

namespace pbp {

  class SlicePBP : public BeliefPropagation {
  public:
    static const int MAX_ACCEPT_ITER = 1000;
    /* default constructor. */
    SlicePBP(VisitingSchedule& visitingSchedule,
             Messages& nlMsg,
             Beliefs& nlBel,
             GraphicalModel& graphicalModel,
             StateSpace& stateSpace,
             GMPotential& gmPotential);
    /* run single Belief Propagation iteration */
    void iterate();

    void setTemperature(double value);
    void setNumberOfMCMCIterations(Index value);
  private:
    double temperature;
    Index nIterMCMC;
    StateSpace stateSpaceNew;
    
    struct MessagePassing
    {
      typedef std::default_random_engine::result_type seed_type;
      MessagePassing(SlicePBP& parent, const Node s, const Index p, const Index VindStart, const Index nNeighbors, const Index nEdges, std::exception_ptr& exception, seed_type seed) :
        parent(parent), s(s), p(p), VindStart(VindStart), nNeighbors(nNeighbors), nEdges(nEdges), exception(exception), random_engine(seed) {};
      void run();
    private:
      SlicePBP& parent;
      std::exception_ptr& exception;
      const Node s;
      const Index p;
      const Index VindStart;
      const Index nNeighbors;
      const Index nEdges;
      std::default_random_engine random_engine;
    };
  };

}
#endif
