/* StateSpace.hpp
 * 
 * Copyright: Oliver Müller (TNT, Leibniz Universität Hannover), 2013
 * E-Mail: omueller@tnt.uni-hannover.de
 */
#ifndef PBP_STATE_STACE_HPP
#define PBP_STATE_STACE_HPP

#include <vector>
#include <pbpTypes.hpp>
#include <GMPotential.hpp>
#include <GraphicalModel.hpp>

namespace pbp {

  typedef Array<Label> LabelSpace;

  class StateSpace {
  public:
    StateSpace(StateSpace& stateSpace);
    StateSpace(GraphicalModel& graphicalModel, Index nDims, Index nParticles);
    StateSpace(GraphicalModel& graphicalModel, LabelSpace &labels);
    StateSpace(GraphicalModel& graphicalModel, std::vector<Index> &nDimsList, std::vector<Index> &nParticlesList);
    StateSpace(GraphicalModel& graphicalModel, Array<LabelSpace> stateSpaceBuffer);

    /* access to variables: */
    LabelSpace& operator[](const Index &index) { return stateSpaceVector[index]; };
    Array<LabelSpace>& operator()() { return stateSpaceVector; };
    Label& operator()(const Index &index, const Index &particle) { return stateSpaceVector[index][particle]; };

    GraphicalModel& getGraphicalModel() const { return graphicalModel; };
    const Index getNumberOfNodes() const { return graphicalModel.getNumberOfNodes(); };
    const Index getNumberOfEdges() const { return graphicalModel.getNumberOfEdges(); };
    const Index getNumberOfParticles(const Index &index) const {return nParticlesList[index];};
    const Index getMaxNumberOfParticles() const;
    
    void copyFrom(StateSpace &stateSpace);

  private:
    GraphicalModel& graphicalModel;
    Array<LabelSpace> stateSpaceVector;
    std::vector<Index> nDimsList;
    std::vector<Index> nParticlesList;

    void createBuffer();
  };

}

#endif
