/* mexPBPclasses.hpp
 * 
 * Copyright: Oliver Müller (TNT, Leibniz Universität Hannover), 2013
 * E-Mail: omueller@tnt.uni-hannover.de
 */
#ifndef PBP_MEX_PBP_CLASSES_HPP
#define PBP_MEX_PBP_CLASSES_HPP

#include <vector>
#include <stdexcept>
#include <mex.h>

#include <pbpTypes.hpp>
#include <GraphicalModel.hpp>
#include <VisitingSchedule.hpp>
#include <SlicePBP.hpp>
#include <GMPotential.hpp>

class mexEdges : public pbp::Array<pbp::GraphicalModel::Edge> {
public:
  mexEdges(const mxArray *edges);
private:
  pbp::Array<pbp::GraphicalModel::Edge> createEdge(const mxArray *edges);
};

class mexSchedule: public pbp::VisitingSchedule {
public:
  mexSchedule(const mxArray *schedules);
private:
  pbp::Array<pbp::Nodes> createSchedule(const mxArray *schedules);
};

class mexStateSpace: public pbp::StateSpace {
public:
  mexStateSpace(pbp::GraphicalModel& graphicalModel, const mxArray *inBuf, mxArray** outBuf);
private:
  pbp::Array<pbp::LabelSpace> createBuf(const mxArray *inBuf, mxArray** outBuf);
};

class mexBeliefs: public pbp::Beliefs {
public:
  mexBeliefs(pbp::StateSpace stateSpace, const mxArray *inBuf, mxArray** outBuf);
private:
  pbp::BeliefBuffer createOutBuf(const mxArray *inBuf, mxArray** outBuf);
  pbp::BeliefBuffer createBuf(const mxArray *bufMat);
};

class mexMessages: public pbp::Messages {
public:
  mexMessages(pbp::StateSpace stateSpace, const mxArray *inBuf, mxArray** outBuf);
private:
  pbp::MessageBuffer createOutBuf(const mxArray *inBuf, mxArray** outBuf);
  pbp::MessageBuffer createBuf(const mxArray *bufMat);
};

class mexTemperatureSchedule: public pbp::Array<double> {
public:
  typedef pbp::Array<double> Buffer;
  mexTemperatureSchedule(const mxArray *inBuf);
private:
  Buffer createBuf(const mxArray *bufMat);
};

#endif
