% function compileGMmex(mexName, libpbpPath)
% function compileGMmex(mexName, libpbpPath, sources)
% 
% mexName: Name of generated MEX file (without file extension)
% libpbpPath: Path to PBP library (e.g. '/usr/lib/pbp')
% sources: String or cell array of strings determining additional *.cpp
%          files to compile and link with
% 
% Usage example:
% >> generateGMPotentialHeader('foo.hpp', 'fooGMpot', sym('x^2'), sym('(x-y)^2'));
% >> generateGMmex('fooBP', 'foo.hpp', 'fooGMpot', []);
% >> compileGMmex('fooBP', '/usr/lib/pbp');
% 
% Copyright: Oliver Müller (TNT, Leibniz Universität Hannover), 2013
function compileGMmex(mexName, libpbpPath, sources)

  if nargin < 3 || isempty(sources)
    sources = {};
  end
  if ~iscell(sources)
    sources = {sources};
  end

  % Compile with mex (and link with pbp library):
  sourceFile = [mexName, '.cpp'];
  libpbp_include = sprintf('-I%s', fullfile(libpbpPath, 'include'));
  libpbp_source = sprintf('-L%s', fullfile(libpbpPath, 'lib'));
  libpbp_matlab = sprintf('-L%s', fullfile(libpbpPath, 'matlab'));
  mex(libpbp_include, ...
    libpbp_source, ...
    libpbp_matlab, ...
    '-lmexpbp', '-lpbp', '-lgomp', ...
    'CFLAGS="\$CFLAGS\ -fopenmp"', ...
    'CXXFLAGS="\$CXXFLAGS\ -fopenmp"', ...
    'LDFLAGS="\$LDFLAGS\ -fopenmp"', ...
    '-v', ...
    sourceFile, sources{:});

end
