#ifndef PBP_CUSTOM_FOO_HPP
#define PBP_CUSTOM_FOO_HPP

#include <stdexcept>
#include "math.h"
#include <Interval.hpp>
#include <GMPotential.hpp>

#define sqr(a) ((a)*(a))

class fooGMpot : public pbp::GMPotential {
public:
  fooGMpot() {};
  virtual pbp::Energy nodePotential(pbp::Label x, pbp::Index n) const {
    return x[0]*x[0];
  };
  virtual pbp::Interval nodeInterval(pbp::Label x, pbp::Index n, pbp::Index dimension, pbp::Energy U) const {
    double left, right;
    pbp::Interval bounds;
    switch (dimension) {
      case 0: 
        if ((U < 0.0)) {
          return pbp::Interval();
        } else
        if ((0.0 <= U)) {
          left = -sqrt(U);
          right = sqrt(U);
          bounds.set_union(left, right);
        } else
          return pbp::Interval();
        break;
      default: throw std::runtime_error("GMPotential: currDim > 0.");
    }
    return bounds;
  };
  virtual pbp::Energy edgePotential(pbp::Label x, pbp::Label y, pbp::Index e) const {
    return sqr(x[0]-y[0]);
  };
  virtual pbp::Interval edgeXInterval(pbp::Label x, pbp::Label y, pbp::Index e, pbp::Index dimension, pbp::Energy U) const {
    double left, right;
    pbp::Interval bounds;
    switch (dimension) {
      case 0: 
        if ((U < 0.0)) {
          return pbp::Interval();
        } else
        if ((0.0 <= U)) {
          left = y[0]-sqrt(U);
          right = y[0]+sqrt(U);
          bounds.set_union(left, right);
        } else
          return pbp::Interval();
        break;
      default: throw std::runtime_error("GMPotential: currDim > 0.");
    }
    return bounds;
  };
  virtual pbp::Interval edgeYInterval(pbp::Label x, pbp::Label y, pbp::Index e, pbp::Index dimension, pbp::Energy U) const {
    double left, right;
    pbp::Interval bounds;
    switch (dimension) {
      case 0: 
        if ((U < 0.0)) {
          return pbp::Interval();
        } else
        if ((0.0 <= U)) {
          left = x[0]-sqrt(U);
          right = x[0]+sqrt(U);
          bounds.set_union(left, right);
        } else
          return pbp::Interval();
        break;
      default: throw std::runtime_error("GMPotential: currDim > 0.");
    }
    return bounds;
  };
};

#endif
